/**************************************************
opengate server
 module for Communication through CGI 

Copyright (C) 1999 Opengate Project Team
Written by Yoshiaki Watanabe
Modified Katsuhiko Eguchi, 2005 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp

Programmed by Yoshiaki WATANABE
Modified by Shin-ichi TADAKI
Modified by Katsuhiko Eguchi
**************************************************/

#include	"opengatesrv.h"

/* convert two-char-hex "aa" to one-number 0Xaa */ 
#define hex2num(x)  ((x)>='A' ? ((x) & 0XDF) - 'A' +10 : ((x) - '0'))

void split(char content[], char *name[], char *value[], char *next[]);
void decode(char *string);

char language[WORDMAXLN]; /* message language in java applet */

/*******************************/
/* get the client addr         */
/*******************************/
void getClientAddr(char *clientAddr)
{
  strncpy(clientAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
}

/********************************************/
/* get Post data from the client  */
/********************************************/
int getPostData(char *userid, char *password, char *clientAddr4, int *durationPtr)
{
  int contentLen;
  char content[BUFFMAXLN];
  char *name[1];
  char *value[1];
  char *next[1];
  char *ptr;
  int item;
  char durationStr[WORDMAXLN];
  char langList[BUFFMAXLN];
  char encodeAddr4[ADDRMAXLN];
  char accessAddr[ADDRMAXLN];
  int ret;

  /* get content sent from web input */
  if(getenv("CONTENT_LENGTH")==NULL)return FALSE;

  contentLen=atoi(getenv("CONTENT_LENGTH"));
  if(contentLen==0)return FALSE;

  contentLen++; /* for terminate ch */
  if(contentLen > BUFFMAXLN) contentLen=BUFFMAXLN;
  if(fgets(content, contentLen, stdin) == NULL){
    content[0]='\0';
  }

  /* get items from string */
  userid[0]='\0';
  password[0]='\0';
  encodeAddr4[0]='\0';
  language[0]='\0';
  durationStr[0]='\0';

  ptr=content;

  while(ptr!=NULL){
    split(ptr, name, value, next);

    if(strstr(name[0], "userid")!=NULL){
      strncpy(userid, value[0], USERMAXLN);
    }else if(strstr(name[0], "password")!=NULL){
      strncpy(password, value[0], PASSMAXLN);
    }else if(strstr(name[0],"remote_addr")!=NULL){
      strncpy(encodeAddr4,value[0],ADDRMAXLN);
    }else if(strstr(name[0], "language")!=NULL){
      strncpy(language, value[0], WORDMAXLN);
    }else if(strstr(name[0], "duration")!=NULL){
      strncpy(durationStr, value[0], WORDMAXLN);
    }
    ptr=next[0];
  }

  /* decode the HTTP encoding */
  decode(userid);
  decode(password);
  decode(encodeAddr4);
  decode(language);
  decode(durationStr);

 /* if not available language, use first lang */
  strncpy(langList, GetConfValue("HtmlLangs"), BUFFMAXLN);  /* list of available languages */
  if(strstr(langList,language)==NULL){
    sscanf(langList,"%s",language);
  }
  
  /* convert duration string to interger and minutes to seconds */
  *durationPtr = atoi(durationStr)*60;
  
  /* encoded address starting as "0-0-0" means no addr info */
  /* it indicates needless to get dual stack addresses */
  /* and only use getenv("REMOTE_ADDR") address */
  if(strnstr(encodeAddr4, "0-0-0", ADDRMAXLN)==encodeAddr4){
    clientAddr4[0]='\0';
  }
      
  /* decode client address to dot separated form */
  else if(AddrDecode(clientAddr4, encodeAddr4)==1){
    /* if can't decode, retry */
    return FALSE;
  }

  /* if the decoded IPv4 addr is not same as access IPv4 addr, use later */
  strncpy(accessAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
  if((strnstr(accessAddr, ".", ADDRMAXLN)!=NULL)   /* access is IPv4 */
     && strncmp(accessAddr, clientAddr4, ADDRMAXLN)!=0){  /* and not same */
    strncpy(clientAddr4, accessAddr, ADDRMAXLN);
  }
  return TRUE;
}


/*********************************************/
/* deny message to the client            */
/*********************************************/
void putClientDeny(char *clientAddr4)
{
  FILE *fp;
  char denydoc[BUFFMAXLN];
  char authCgiUrl[BUFFMAXLN];
  char encodeAddr[ADDRMAXLN];

  /* keyword pairs */
  /*  the left key is replaced by the right value */
  struct html_key keys[]=
    {
      {"%%AUTHCGIURL%%", authCgiUrl},
      {"%%ADDR4%%", encodeAddr},
      {"",""}  /* DON'T REMOVE THIS LINE */
    };
 
  /* create authcgi URL string */
  snprintf(authCgiUrl, BUFFMAXLN, "%s%s%s/%s",
	   GetConfValue("OpengateServerName"),
	   GetConfValue("CgiDir"),
	   GetConfValue("OpengateDir"),
	   GetConfValue("AuthCgi"));

  /* create encoded addr4 */
  if(AddrEncode(encodeAddr, clientAddr4)==1){
    encodeAddr[0]='\0';
  }

  /* make path to the denydoc for ssl or non-ssl */
  if(strcmp(getenv("SERVER_PORT"),GetServicePortStr("https"))==0){
    sprintf(denydoc,"%s%s/%s/%s",GetConfValue("DocumentRoot"),
	    GetConfValue("OpengateDir"),language,GetConfValue("DenyDocSsl"));
  }else{
    sprintf(denydoc,"%s%s/%s/%s",GetConfValue("DocumentRoot"),
	    GetConfValue("OpengateDir"),language,GetConfValue("DenyDoc"));
  }

  /* replace keyword and send out the file */
  printf("Content-type: text/html\r\n\r\n\r\n");
  HtmlTemplate(denydoc, keys);

  return;
}
/*********************************************/
/* deny message to the client            */
/*********************************************/
void putClientRetry(char *lang)
{
  FILE *fp;
  char retrydoc[BUFFMAXLN];
  char externalUrl[BUFFMAXLN];
  char authCgiUrl[BUFFMAXLN];

  /* keyword pairs */
  /*  the left key is replaced by the right value */
  struct html_key keys[]=
    {
      {"%%EXTERNALURL%%", externalUrl},
      {"%%AUTHCGIURL%%", authCgiUrl},
      {"",""}  /* DON'T REMOVE THIS LINE */
    };
 
  /* create external URL string */
  strncpy(externalUrl, GetConfValue("ExternalUrl"), BUFFMAXLN);

  /* create authcgi URL string */
  snprintf(authCgiUrl, BUFFMAXLN, "%s%s%s/%s",
	   GetConfValue("OpengateServerName"),
	   GetConfValue("CgiDir"),
	   GetConfValue("OpengateDir"),
	   GetConfValue("AuthCgi"));

  /* make read in path to the retry document */
  sprintf(retrydoc,"%s%s/%s/%s",GetConfValue("DocumentRoot"),
	  GetConfValue("OpengateDir"),lang,GetConfValue("RetryDoc"));

  /* replace keyword and send out the file */
  printf("Content-type: text/html\r\n\r\n\r\n");
  HtmlTemplate(retrydoc, keys);

  return;
}

/*********************************************/
/* put some message to the client            */
/*********************************************/
void putClientMsg(char *message)
{
  printf("Content-type: text/html\r\n\r\n\r\n");
  printf("<HTML><HEAD><TITLE>OpengateMsg</TITLE></HEAD> \r\n");
  printf("<BODY>         \r\n");
  printf("%s\r\n",     message);
  printf("</BODY></HTML> \r\n\r\n");

}

/*********************************************/
/* put accept message and java to the client */
/*********************************************/
void putJavaApplet(char *userid, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus)
{
  FILE *fp;
  char buff[BUFFMAXLN];
  int markfound=0;
  char acceptdoc[BUFFMAXLN];
  char acceptdoc2url[BUFFMAXLN];
  char *ptr;
  char terminateurl[BUFFMAXLN];

  char *startPageUrl=GetConfValue("StartPage/Url");
  int startPageType=atoi(GetConfValue("StartPage/Type"));
  char *opengateDir=GetConfValue("OpengateDir");

  /* create path to acceptdoc */
  sprintf(acceptdoc,"%s%s/%s/%s",GetConfValue("DocumentRoot"),
	  GetConfValue("OpengateDir"),language,GetConfValue("AcceptDoc"));
  sprintf(acceptdoc2url, 
	  "http://%s%s/%s/%s",GetConfValue("OpengateServerName"),
	  GetConfValue("OpengateDir"),language,GetConfValue("AcceptDoc2"));

  /* create terminate url [http://<servaddr>:<port>/terminate<pid>] */
  sprintf(terminateurl, "http://%s:%d/terminate%d", 
	  GetConfValue("OpengateServerName"), port, pid);

  /* open acceptdoc */
  if((fp=fopen(acceptdoc, "r"))==NULL){
    err_msg("ERR at %s#%d: cannot open %s",__FILE__,__LINE__,acceptdoc);
    PutClientMsg("Cannot find html document");
    return;
  }

  /* read html document from file and send to web */
  printf("Content-type: text/html\r\n\r\n\r\n");
  while(fgets(buff, BUFFMAXLN, fp)!=NULL){

    /* search insert mark and replace it */

    if(strstr(buff,"%%OPENGATEAPPLET%%")!=NULL){
      /* applet insert mark found */

      markfound=1;
      printf("<applet code='Opengate.class' archive='Opengate.jar' ");
      printf("codebase='%s' width=600 height=30 > \r\n", opengateDir );
      printf("<param name=port value='%d'>\r\n", port );
      printf("<param name=user value='%s'>\r\n", userid );
      printf("<param name=lang value='%s'>\r\n", language );
      printf("</applet>                   \r\n");

      if(ipStatus==IPV4ONLY){
        printf("<table border=0>\r\n");
        printf("<tr><td>\r\n");
        printf("IPv4 address : [%s]\r\n",clientAddr4);
        printf("</td></tr>\r\n");
        printf("</table>\r\n");
      }else if(ipStatus=IPV46DUAL){
        printf("<table border=0>\r\n");
        printf("<tr><td>\r\n");
        printf("IPv4 address : [%s]\r\n",clientAddr4);
        printf("</td></tr>\r\n");
        printf("<tr><td>\r\n");
        printf("IPv6 address : [%s]\r\n",clientAddr6);
        printf("</td></tr>\r\n");
        printf("</table>\r\n");
      }else if(ipStatus==IPV6ONLY){
        printf("<table border=0>\r\n");
        printf("<tr><td>\r\n");
        printf("IPv6 address : [%s]\r\n",clientAddr6);
        printf("</td></tr>\r\n");
        printf("</table>\r\n");
      }

    }else{
      /* replace terminate url mark */
      htmlReplace(buff, "%%TERMINATEURL%%", terminateurl);

      /* replace information url mark */
      if( startPageType==1 ){
      	htmlReplace(buff, "%%STARTURL%%", startPageUrl);
      }else{
	htmlReplace(buff, "%%STARTURL%%", acceptdoc2url);
      }

      printf("%s",buff);
    }
  }
  fputs("\r\n\r\n",stdout);
  fclose(fp);

  if(markfound==0){
    err_msg("ERR at %s#%d: cannot find applet insert mark in %s",
                   __FILE__,__LINE__,acceptdoc);
    PutClientMsg("Cannot find applet mark");
  }

  return;
}

/************************************/
/* split value for indicated name   */
/* in content  "name=value&..."     */
/************************************/
void split(char content[], char *name[], char *value[], char *next[])
{
  char *pstr;
  
  /* set default */
  name[0]=content;
  value[0]=content+strlen(content);
  next[0]=value[0];

  /* set name end */
  if((pstr=strchr(name[0],(int)'='))==NULL){
    next[0]=NULL;
    return;
  }
  *pstr='\0';
  
  /* set value start */
  pstr++;
  value[0]=pstr;
  
  /* set value end */
  if((pstr=strchr(value[0],'&'))==NULL){
    next[0]=NULL;
    return;
  }
  *pstr='\0';

  /* set next start */
  pstr++;
  next[0]=pstr;

  return;
}

/**********************************/
/* decode text coding in web post */
/**********************************/
void decode(char *string)
{
  char *pcheck, *pinsert;

  pcheck=pinsert=string;
  while(*pcheck != '\0'){
    if(*pcheck == '+'){
      *pinsert = ' ';
    }else if(*pcheck == '%'){
      *pinsert=(char)(hex2num(*(pcheck+1))*16 + hex2num(*(pcheck+2)));
      pcheck+=2;
    }else{
      *pinsert=*pcheck;
    }
    pcheck++;
    pinsert++;
  }
  *pinsert=*pcheck;
}

/***************************************/
/* get HTTP_REFERER and check true url */
/***************************************/
int checkReferer(void)
{
  char url[BUFFMAXLN]="";
  if(getenv("HTTP_REFERER")!=NULL){
    strncpy(url,getenv("HTTP_REFERER"),BUFFMAXLN);
    if(strstr(url,GetConfValue("OpengateServerName"))==NULL){
      return FALSE;
    }
  }
  return TRUE;
}

/*******************************/
/*******************************/
void GetClientAddr(char *clientAddr)
{
  if(debug) err_msg("DEBUG:=>getClientAddr( )");
  getClientAddr(clientAddr);
  if(debug) err_msg("DEBUG:<=getClientAddr(%s)",clientAddr);
}


int GetPostData(char *userid, char *password, char *clientAddr4, int *durationPtr)
{
  int ret;

  if(debug) err_msg("DEBUG:=>getPostData( )");
  ret=getPostData(userid,password,clientAddr4,durationPtr);
  if(debug) err_msg("DEBUG:%d<=getPostData(%s,passwd,%s,%d)",ret,userid,clientAddr4,*durationPtr);
  return ret;
}

void PutJavaApplet(char *userid, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus)
{
  if(debug) err_msg("DEBUG:=>putJavaApplet(%s,%d,%d,%s,%s,%d)",userid,port,pid,clientAddr4,clientAddr6,ipStatus);
  putJavaApplet(userid,port,pid,clientAddr4,clientAddr6,ipStatus);
  if(debug) err_msg("DEBUG:<=putJavaApplet( )");
}

void PutClientDeny(char *clientAddr4)
{
  if(debug) err_msg("DEBUG:=>putClientDeny(&s)",clientAddr4);
  putClientDeny(clientAddr4);
  if(debug) err_msg("DEBUG:<=putClientDeny( )");
}

void PutClientRetry(char *lang)
{
  if(debug) err_msg("DEBUG:=>putClientRetry(%s)",lang);
  putClientRetry(lang);
  if(debug) err_msg("DEBUG:<=putClientRetry( )");
}

void PutClientMsg(char *message)
{
  if(debug) err_msg("DEBUG:=>putClientMsg( %s )",message);
  putClientMsg(message);
  if(debug) err_msg("DEBUG:<=putClientMsg( )");
}

int CheckReferer(void)
{
  int ret;
  if(debug) err_msg("DEBUG:=>checkReferer( )");
  ret = checkReferer();
  if(debug) err_msg("DEBUG:(%d)<=checkReferer( )",ret);
  return ret;
}
