/**************************************************
Opengate authentication CGI main

This program is accessed as [http(s)://xx.yy/opengateauth.cgi?ja]
It send out the authentication page after keywords replacement.

Copyright (C) 2005 Opengate Project Team
Written by Katsuhiko Eguchi, 2005 
Modified by Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include "opengatesrv.h"

int  main(int argc, char **argv)
{

  char htmlFile[BUFFMAXLN]="";   /* html file */
  char lang[WORDMAXLN]="";       /* language */
  char referer[BUFFMAXLN]="";    /* HTTP referer */
  char clientAddr[ADDRMAXLN]=""; /* client ip address */
  char paramString[BUFFMAXLN]="";   /* parameters in html access */
  char* pLang;  /* pointer to language string */ 
  char* pAddr4; /* pointer to addr4 string */
  char authCgiUrl[BUFFMAXLN];  /* url of opengateauth.cgi */
  char mainCgiUrl[BUFFMAXLN];  /* url of opengatesrv.cgi */
  char remoteAddr[ADDRMAXLN];  /* remote address */
  char durationMax[WORDMAXLN]; /* usage duration maximum */

  /* keyword pairs */
  /*  the left key is replaced by the right value */
  struct html_key keys[]=
    {
      {"%%AUTHCGIURL%%", authCgiUrl},
      {"%%CGIURL%%", mainCgiUrl},
      {"%%ADDR4%%", clientAddr},
      {"%%DURATIONMAX%%",durationMax},
      {"",""}  /* DON'T REMOVE THIS LINE */
    };

  /* prepare config file */
  if(OpenConfFile()==-1) return 0;
 
  /* start log */
  errToSyslog(atoi(GetConfValue("Syslog/Enable")));
  openlog(GetConfValue("AuthCgi"), LOG_PID, atoi(GetConfValue("Syslog/Facility")));

  /* initialize config */
  InitConf();

  if(debug) err_msg("DEBUG: started");

  /* create URL string */
  snprintf(authCgiUrl, BUFFMAXLN, "%s%s%s/%s",
	   GetConfValue("OpengateServerName"),
	   GetConfValue("CgiDir"),
	   GetConfValue("OpengateDir"),
	   GetConfValue("AuthCgi"));
  snprintf(mainCgiUrl, BUFFMAXLN, "%s%s%s/%s",
	   GetConfValue("OpengateServerName"),
	   GetConfValue("CgiDir"),
	   GetConfValue("OpengateDir"),
	   GetConfValue("MainCgi"));
  
  /* setup usage duration maximum (save as seconds, display as minutes */
  snprintf(durationMax,WORDMAXLN,"%d",atoi(GetConfValue("Duration/Max"))/60);

  /* if has paremeters, */
  if(getenv("QUERY_STRING")!=NULL){

    /* get html access parameter string (xx.cgi?addr4&lang) */
    strncpy(paramString, getenv("QUERY_STRING"), BUFFMAXLN);

  }else{    
    /* if not get param, set as null */
    paramString[0]='\0';
  }

  /* split language and address */
  pAddr4=paramString;
  pLang=strnstr(paramString, "&", BUFFMAXLN);
  if(pLang!=NULL){
    *pLang='\0';
    pLang++;
  }

  /* copy clientAddr(encoded) */
  if(isNull(pAddr4)){
    clientAddr[0]='\0';
  }else{
    strncpy(clientAddr, pAddr4, ADDRMAXLN);
  }

  /* get language and check its correctness */
  if(!isNull(pLang) && strstr(GetConfValue("HtmlLangs"), pLang)!=NULL){

    /* if corrrect, set it */
    strncpy(lang, pLang, WORDMAXLN);
  }else{

    /* if not correct, get default language at the top of lang list */
    sscanf(GetConfValue("HtmlLangs"), "%s", lang);
  }

  /* send out header */
  printf("Content-Type: text/html\r\n\r\n\r\n");

  /* construct html file path */
  sprintf(htmlFile, "%s%s/%s/", GetConfValue("DocumentRoot"),
	  GetConfValue("OpengateDir"), lang);

  /* ssl or non-ssl file */
  if(strcmp(getenv("SERVER_PORT"),GetServicePortStr("https"))==0) {
    strncat(htmlFile, GetConfValue("AuthDocSsl"), BUFFMAXLN);
  }else{
    strncat(htmlFile, GetConfValue("AuthDoc"), BUFFMAXLN);
  }

  /* replace keywords and send out */
  HtmlTemplate(htmlFile, keys);

  if(debug) err_msg("DEBUG: terminated");  

  return 0;
}
